/***************************************************************************/
/* biased.c								   */
/*									   */
/* Generator of biased probability bits.				   */
/*									   */
/* usage:  rannos | biased [n] | ....					   */
/*									   */
/* n is a value from 1 to 99, giving the percentage of 0's in the stream.  */
/*									   */
/* stdin is assumed to be true random bytes.				   */
/***************************************************************************/

#include <stdio.h>
#include <stdlib.h>

unsigned long ranmod(unsigned long R)
{
  unsigned long x = 256 % R ;
  unsigned long y ;

  if ( R < 2 ) return ( 0 ) ;   /* no reason to get ranval() */
  if ( R > 256 ) exit ( 1 ) ;	/* the loop will be infinite */
  do { y = getchar() ; } while (y < x) ;

  return ( (y-x) % R ) ;
} /* ranmod */

unsigned char eight_bits( int pct )
{
  int i ;
  unsigned char c = 0 ;

  for (i=0; i<8; i++) {
    c <<= 1 ;			/* shift it up by 1 bit */
    if (ranmod( 100 ) < pct)
      c |= 1 ;			/* add in a 1 */
  } /* for */
  return ( c ) ;
} /* eight_bits */

int main(int argc, char **argv)
{
  int pct ;

  if (argc != 2) {
    printf( "Usage: %s <\% of 1s as integer>\n", argv[0] ) ;
    return ( 1 ) ;
  } /* if */

  pct = atoi( argv[1] ) ;

  while ((!feof(stdin)) && (!ferror(stdin)))
    putchar( eight_bits( pct ) ) ;

  return ( 0 ) ;
} /* main */
