/***************************************************************************/
/* Ueli Maurer's Universal Statistical Test, in C			   */
/***************************************************************************/

#include <math.h>
#include <stdio.h>

#define L (8)			/* greater than # bits of memory in
				   the source -- width of our test block */
#define V (256)			/* 2^L */
#define Q (2000)		/* initialization sample */
#define K (40000)		/* tested sample */

int main( void )		/* no parameters */
{
  long int i, n, nsamp ;	/* loop vbls */
  double sum, ftu, dk, ent, ranhn, ranhx ; /* test results */
  long int tab[V] ;		/* last location of this value */

  printf( "Note: the estimate being gathered from %d bytes of stdin\n\
is valid only if the device producing those samples has\n\
fewer than 8 bits of internal memory.\n\n", Q+K ) ;

  for (i=0; i<V; i++) tab[i] = 0 ;
  for (n=0; n<Q; n++) tab[ source() ] = n ;
  sum = 0.0 ;			/* start the test */
  nsamp = 0 ;
  for (n=Q; n<(Q+K); n++) {
    int val = source() ;	/* get a test value */
    if (val == EOF) break ;
    sum = sum + log( n - tab[ val ] ) ; /* accumulate log of distances */
    nsamp++ ;
    tab[ val ] = n ;		/* update last position */
  } /* for */
  dk = nsamp ;
  if (nsamp < 100) {
    printf( "Far too few samples for an estimate.\n" ) ;
    return (1) ;
  } else 
    printf( "Estimate from %d samples (%d preferred).\n", nsamp, K ) ;
  ftu = sum/( dk*log( 2.0 ) ) ;
  ent = ftu + 0.832746 ;	/* his magic constant, in the limit */
  printf( "Estimated entropy of a byte = %5.2f bits\n", ent ) ;
  ranhn = 3.0 + (16.0*8.0/ent) ; /* # of bytes in a batch */
  if (ranhn < 20.0) ranhn = 20.0 ;
  ranhx = 1.5 * ranhn ;
  printf( "recommend: ranH -n %.0f -x %.0f\n", ranhn-16.0, ranhx-16.0 ) ;
  return ( 0 ) ;
} /* main */

int source( void )
{
  return ( getchar() ) ;
} /* source */
